import React, { useRef } from 'react';
import Link from 'next/link'
import { Box, Button } from "@chakra-ui/react"
import { AddIcon } from '@chakra-ui/icons';
import { encode } from "blurhash";
import useStore from './store';

function Header() {
  const uploader = useRef(null);
  const { ownerToken, firstName, loggedIn, setLoggedIn, authToken, setAuthToken, addPhoto, setIsUploading, myPhotos, setMyPhotos } = useStore();

  const logout = () => {
    if (window.confirm('Really logout?')) {
      setAuthToken('');
      setLoggedIn(false);
    }
  }

  const resizeImage = file => {
    if (window.File && window.FileReader && window.FileList && window.Blob) {
      return new Promise((resolve, reject) => {
        var reader = new FileReader();
        reader.onload = function(e) {
          var img = new Image();
          img.onload = () => {
            var canvas = document.createElement("canvas");
            var ctx = canvas.getContext("2d");
            ctx.drawImage(img, 0, 0);

            var MAX_WIDTH = 400;
            var MAX_HEIGHT = 400;
            var width = img.width;
            var height = img.height;

            if (width > height) {
              if (width > MAX_WIDTH) {
                height *= MAX_WIDTH / width;
                width = MAX_WIDTH;
              }
          } else {
              if (height > MAX_HEIGHT) {
                width *= MAX_HEIGHT / height;
                height = MAX_HEIGHT;
              }
            }
            canvas.width = width;
            canvas.height = height;
            ctx.drawImage(img, 0, 0, width, height);
            const imageData = ctx.getImageData(0, 0, width, height);
            const hash = encode(imageData.data, imageData.width, imageData.height, 4, 4);
            const dataurl = canvas.toDataURL(file.type);

            canvas.toBlob(function(blob) {
              const resizedFile = new File([blob], file.name);
              console.log(resizedFile);
              resolve({ resizedFile, blurHash: hash });
            }, 'image/jpeg', 1);
          }
          img.src = e.target.result;
        }
        reader.readAsDataURL(file);
      });
    } else {
        alert('The File APIs are not fully supported in this browser.');
    }
  }

  const getImageData = file => {
    const canvas = document.createElement("canvas");
    const context = canvas.getContext("2d");
    var image = new Image;
    const myPromise = new Promise((resolve, reject) => {
      image.onload = () => {
        canvas.width = image.width;
        canvas.height = image.height;
        context.drawImage(image, 0, 0);
        resolve(context.getImageData(0, 0, image.width, image.height));
      }
    });
    image.src = URL.createObjectURL(file);
    return myPromise;
  };

  const getVideoCover = (file, seekTo = 0.0) => {
    return new Promise((resolve, reject) => {
        // load the file to a video player
        const videoPlayer = document.createElement('video');
        videoPlayer.setAttribute('src', URL.createObjectURL(file));
        videoPlayer.load();
        videoPlayer.addEventListener('error', (ex) => {
            reject("error when loading video file", ex);
        });
        // load metadata of the video to get video duration and dimensions
        videoPlayer.addEventListener('loadedmetadata', () => {
            // seek to user defined timestamp (in seconds) if possible
            if (videoPlayer.duration < seekTo) {
                reject("video is too short.");
                return;
            }
            // delay seeking or else 'seeked' event won't fire on Safari
            setTimeout(() => {
              videoPlayer.currentTime = seekTo;
            }, 200);
            // extract video thumbnail once seeking is complete
            videoPlayer.addEventListener('seeked', () => {
                console.log('video is now paused at %ss.', seekTo);
                // define a canvas to have the same dimension as the video
                const canvas = document.createElement("canvas");
                canvas.width = videoPlayer.videoWidth;
                canvas.height = videoPlayer.videoHeight;
                // draw the video frame to canvas
                const ctx = canvas.getContext("2d");
                ctx.drawImage(videoPlayer, 0, 0, canvas.width, canvas.height);
                // return the canvas image as a blob
                ctx.canvas.toBlob(
                    blob => {
                        resolve(blob);
                    },
                    "image/jpeg",
                    0.75 /* quality */
                );
            });
        });
    });
  }

  const uploadFile = async (file) => {
    if (!file) return;
    const isImage = file.type.split('/')[0] === 'image';
    const isVideo = file.type.split('/')[0] === 'video';
    if (!isImage && !isVideo) return alert('Please select an image or video.');

    let blurHash, resizedFile;
    if (isImage) {
      const resizeData = await resizeImage(file);
      blurHash = resizeData.blurHash;
      resizedFile = resizeData.resizedFile;
      if (!blurHash || !resizedFile) return;
    }
    let videoThumb;
    if (isVideo) {
      videoThumb = await getVideoCover(file, 1.5);
    }

    const uploadRequestResponse = await fetch(`/api/upload?name=${file.name}&type=${file.type}`, {
      headers: { Authorization: authToken },
    });
    if (uploadRequestResponse.ok) {
      const responseJson = await uploadRequestResponse.json();

      // If server offers chance to upload original file, then do so in background
      if (responseJson.originalUrl) {
        fetch(responseJson.originalUrl, { method: 'PUT', body: file });
      }
      if (responseJson.thumbUrl) {
        fetch(responseJson.thumbUrl, { method: 'PUT', body: videoThumb });
      }

      const uploadResponse = await fetch(responseJson.url, {
        method: 'PUT',
        body: resizedFile || file,
      });
      if (uploadResponse.ok) {
        const storeResponse = await fetch('/api/photos', {
          method: 'POST',
          headers: {
            Authorization: authToken,
            'Content-Type': 'application/json;charset=utf-8'
          },
          body: JSON.stringify({ ownerToken, firstName, fileName: responseJson.fileName, blurHash, isImage, isVideo, originalFileName: responseJson.originalFileName, thumbFileName: responseJson.thumbFileName }),
        });
        const storeResponseJson = await storeResponse.json();
        if (storeResponseJson?.url) {
          addPhoto({ firstName, fileName: responseJson.fileName, src: storeResponseJson.url, thumbSrc: storeResponseJson.thumbUrl, blurHash, isImage, isVideo });
          myPhotos.push(responseJson.fileName);
          setMyPhotos(myPhotos);
          window.localStorage.setItem('myPhotos', JSON.stringify({ photos: myPhotos }));
        }
      }
    }
  }

  const handleFileChosen = async (event) => {
    const files = event.target?.files;
    if (files?.length > 0) {
      setIsUploading(true);
      for (let i = 0; i < files.length; i++) {
        await uploadFile(files[i]);
      }
      setIsUploading(false);
    }
  }

  return (
    <Box bg="teal.400" w="100%" p={4} color="white" d='flex' justifyContent='space-between' alignItems='center'>
      <img src='/logo.png' style={{height: 50}} onClick={logout} />

      {loggedIn && <>
        <Button onClick={() => uploader?.current?.click()} leftIcon={<AddIcon />} colorScheme="teal" variant="solid">
          Add a photo or video
        </Button>
        <input multiple type='file' style={{display:'none'}} ref={uploader} onChange={handleFileChosen} accept='image/*,video/*' /> 
      </>}
    </Box>
  );
}

export default Header;
