import React from 'react';
import { S3Client, GetObjectCommand } from "@aws-sdk/client-s3";
import { getSignedUrl } from "@aws-sdk/s3-request-presigner";
import faunadb from 'faunadb';
import jwt from 'jsonwebtoken';

const q = faunadb.query;
const s3 = new S3Client({
  endpoint: process.env.S3_ENDPOINT,
  credentials: {
    accessKeyId: process.env.ACCESS_KEY_ID,
    secretAccessKey: process.env.SECRET_ACCESS_KEY,
  },
});

function getAudience(req, res) {
  if (req.headers['authorization']) {
    try {
      const decoded = jwt.verify(req.headers['authorization'], process.env.JWT_SECRET);
      return decoded.audience;
    } catch(err) {
      console.log(err);
      res.status(403).json({ message: 'Invalid token' });
    }
  }
  else res.status(403).json({ message: 'Authorization required' });
}

export default async (req, res) => {
  const audience = getAudience(req, res);
  if (!audience) return;

  if (req.method === 'GET') {
    const itemsPerPage = 18;
    
    const params = req.query;
    const next = req.query && parseInt(req.query.next);

    const client = new faunadb.Client({ secret: process.env.FAUNA_SECRET })
    const helper = await client.query(
      q.Paginate(
        q.Match(
          q.Index("reverse_photos")
        ),
        {
          size: itemsPerPage,
          after: next ? [next] : undefined,
        }
      )
    );
    
    const baseUrl = `https://${process.env.S3_BUCKET}.eu-central-1.linodeobjects.com/`;
    const photos = helper.data.map(photo => ({
      ts: photo[0],
      audience: photo[1],
      firstName: photo[2],
      fileName: photo[3],
      src: `${baseUrl}${photo[3]}`,
      origianlFileName: photo[4],
      originalSrc: photo[4] ? `${baseUrl}${photo[4]}` : null,
      thumbFileName: photo[5],
      thumbSrc: photo[5] ? `${baseUrl}${photo[5]}` : null,
      blurHash: photo[6],
      isImage: photo[7],
      isVideo: photo[8],
      refId: photo[9].id,
    }));
    res.status(200).json({ photos, next: helper.after[0], end: !helper.after});
  }

  if (req.method == 'POST') {
    const client = new faunadb.Client({ secret: process.env.FAUNA_SECRET });
    const { ownerToken, firstName, fileName, originalFileName, thumbFileName, blurHash, isImage, isVideo } = req.body;
    const createP = await client.query(
      q.Create(
        q.Collection('photos'),
          { data: { audience, ownerToken, firstName, fileName, originalFileName, thumbFileName, blurHash, isImage, isVideo } }
      )
    );
    res.status(200).json({ url: `https://${process.env.S3_BUCKET}.eu-central-1.linodeobjects.com/${fileName}`, thumbUrl: `https://${process.env.S3_BUCKET}.eu-central-1.linodeobjects.com/${thumbFileName}` });
  }

  if (req.method == 'DELETE') {
    const { ownerToken, id } = req.query;
    const client = new faunadb.Client({ secret: process.env.FAUNA_SECRET });
    const result = await client.query(
      q.Get(q.Ref(q.Collection('photos'), id))
    );

    if (result && result.data.ownerToken && ownerToken && ownerToken === result.data.ownerToken) {
      await client.query(
        q.Delete(q.Ref(q.Collection('photos'), id))
      );
    }
    res.status(200).json({ deleted: id });
  }
}
