import React, { useEffect } from 'react';
import { v4 as uuidv4 } from 'uuid';
import { Text, Grid, GridItem, Box, AspectRatio, Progress, CircularProgress } from '@chakra-ui/react';
import InfiniteScroll from 'react-infinite-scroll-component';
import Layout from '../lib/Layout';
import LoginPage from '../lib/Login';
import BlurrableImage from '../lib/BlurrableImage';
import useStore from '../lib/store';

export default function Home({  }) {
  const { firstName, setFirstName, loggedIn, setLoggedIn, authToken, setAuthToken, ownerToken, setOwnerToken, isUploading, uploadingCount, selectedPhotoIndex, setSelectedPhotoIndex, photos, setPhotos, addPhoto, myPhotos, setMyPhotos, nextPhoto, setNextPhoto, endReached, setEndReached } = useStore();

  const getPhotos = async (refresh) => {
    if (loggedIn && authToken) {
      const response = await fetch(`/api/photos${(!refresh && nextPhoto) ? `?next=${nextPhoto}` : ''}`, {
        headers: {
          Authorization: authToken,
        },
      });
      const json = await response.json();
      setPhotos(refresh ? json.photos : photos.concat(json.photos));
      if (json.next) setNextPhoto(json.next);
      setEndReached(json.end);
    }
  }

  const deletePhoto = async (e, photo) => {
    e.stopPropagation();
    if (window.confirm('Really delete this photo?')) {
      const response = await fetch(`/api/photos?id=${photo.refId}&ownerToken=${ownerToken}`, {
        method: 'DELETE',
        headers: {
          Authorization: authToken,
        },
      });
      const json = await response.json();
      if (json?.deleted) {
        setSelectedPhotoIndex(-1);
        setPhotos(photos.filter(p => p.refId !== photo.refId));
      }
    }
  }

  // One-time function: load photo URLs from local storage on app-load
  useEffect(() => {
    const myPhotosString = window.localStorage.getItem('myPhotos');
    if (myPhotosString) {
      const unencodedString = JSON.parse(myPhotosString);
      if (unencodedString?.photos) {
        setMyPhotos(unencodedString.photos);
      }
    }
    let storedOwnerToken = window.localStorage.getItem('ownerToken');
    if (!storedOwnerToken) {
      storedOwnerToken = uuidv4();
      window.localStorage.setItem('ownerToken', storedOwnerToken);
    }
    setOwnerToken(storedOwnerToken);
    const storedAuthToken = window.localStorage.getItem('authToken');
    if (storedAuthToken) {
      setLoggedIn(true);
      setAuthToken(storedAuthToken);
    }
    const storedFirstName = window.localStorage.getItem('firstName');
    if (storedFirstName) {
      setFirstName(storedFirstName);
    }
  }, []);

  const fetchData = () => {
    getPhotos();
  }

  useEffect(() => {
    window.localStorage.setItem('authToken', authToken);
  }, [authToken]);

  useEffect(() => {
    window.localStorage.setItem('firstName', firstName);
  }, [firstName]);

  // If login status changes then re-fetch photos
  useEffect(getPhotos, [loggedIn, authToken]);

  if (!loggedIn) return <LoginPage />;

  const selectedPhoto = photos && photos[selectedPhotoIndex];
  return (
    <Layout>
      <Box flex={1}>
      {isUploading &&
        <Progress size="sm" isIndeterminate colorScheme='blue' />
      }
        {/*
      <Grid templateColumns="repeat(3, 1fr)" gap={0.5} flex={1}>*/}
 	<InfiniteScroll
            dataLength={photos.length}
            next={fetchData}
            hasMore={!endReached}
            loader={<h4 style={{textAlign: 'center'}}>Loading...</h4>}
            endMessage={
              <p style={{ textAlign: 'center' }}>
                <b>Yay! You have seen it all</b>
              </p>
            }
            style={{display: 'grid', gridGap: 2, gridTemplateColumns: 'repeat(3, 1fr)'}}
          >
            {photos.map((photo, i) =>
              <AspectRatio maxW="100%" ratio={1} key={photo.src}>
                <BlurrableImage {...photo} onClick={e => setSelectedPhotoIndex(i)}/>
              </AspectRatio>
            )}
          </InfiniteScroll>
          {/*</Grid>*/}

      {selectedPhotoIndex > -1 &&
        <div style={{position: 'fixed', top: 0, width: '100%', height: '100%', background: 'rgba(0,0,0,0.8)', display: 'flex', flexDirection: 'column', justifyContent: 'center'}} onClick={e => setSelectedPhotoIndex(-1)}>
          {(selectedPhoto.isImage || !selectedPhoto.isVideo) &&
            <img src={selectedPhoto.src} style={{maxWidth: '100%', maxHeight: '900px', alignSelf: 'center', display: 'block'}} onClick={e => e.stopPropagation()}/>
          }
          {selectedPhoto.isVideo &&
            <video playsInline style={{maxWidth:'100%', maxHeight: '900px', alignSelf: 'center', display: 'block'}} controls autoPlay onClick={e => e.stopPropagation()}>
              <source src={selectedPhoto.src} />
            </video>
          }
          {selectedPhoto.firstName &&
            <Text color='white' textAlign='center' fontSize='lg'  mt={5}>📸 Added by {selectedPhoto.firstName}</Text>
          }

          <div style={{position: 'absolute', left: '5px', top: '10px', fontSize: 15, background: 'rgba(0,0,0,0.5)', padding: 4, borderRadius: '50%', width:30, height: 30, textAlign: 'center' }} onClick={e => {setSelectedPhotoIndex(-1);e.stopPropagation();}}>
            ❌
          </div>
          {myPhotos?.indexOf(selectedPhoto.fileName) > -1 && selectedPhoto.refId &&
            <div style={{position: 'absolute', right: '5px', top: '30%', fontSize: 20, background: 'rgba(0,0,0,0.5)', padding: 4, borderRadius: '50%', width:40, height: 40, textAlign: 'center' }} onClick={e => deletePhoto(e, selectedPhoto)}>
              🚮
            </div>
          }
          {selectedPhoto.originalSrc &&
            <a href={selectedPhoto.originalSrc} target="_blank" rel="noopener noreferrer" style={{position: 'absolute', right: '5px', top: '40%', fontSize: 20, background: 'rgba(0,0,0,0.5)', padding: 4, borderRadius: '50%', width:40, height: 40, textAlign: 'center' }} onClick={e => e.stopPropagation()}>
              🖼
            </a>
          }

          {selectedPhotoIndex > 0 &&
            <div style={{position: 'absolute', left: '5px', top: '45%', fontSize: 20, background: 'rgba(0,0,0,0.5)', padding: 4, borderRadius: '50%', width:40, height: 40, textAlign: 'center' }} onClick={e => {setSelectedPhotoIndex(selectedPhotoIndex - 1);e.stopPropagation();}}>
              👈
            </div>
          }
          {selectedPhotoIndex < photos.length - 1&&
            <div style={{position: 'absolute', right: '5px', top: '45%', fontSize: 20, background: 'rgba(0,0,0,0.5)', padding: 4, borderRadius: '50%', width:40, height: 40, textAlign: 'center' }} onClick={e => {setSelectedPhotoIndex(selectedPhotoIndex + 1);e.stopPropagation();}}>
              👉
            </div>
          }
        </div>
      }
    </Box>
    </Layout>
  )
}
